﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.BinPacking.Dimensions {
  [Item("Packing Dimensions", "Represents an abstract packing-position associated with a packing-problem.")]
  [StorableClass]
  public abstract class PackingDimensions : Item, IPackingDimensions {
    /// <summary>
    /// The number of the bin to which the current item is assigned is stored in this variable. Counting starts at 0 !!!
    /// </summary>
    public Int32 AssignedBin { get; set; }

    [StorableConstructor]
    protected PackingDimensions(bool deserializing) : base(deserializing) { }
    protected PackingDimensions(PackingDimensions original, Cloner cloner)
      : base(original, cloner) {
        AssignedBin = original.AssignedBin;
    }

    public PackingDimensions(Int32 assignedBin) {
      this.AssignedBin = assignedBin;
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[");
      sb.Append("AssignedBin: ");
      sb.Append(AssignedBin);
      sb.Append("]");
      return sb.ToString();
    }
  }
}
